angular.module('Landers.api', [])
    .factory('AngularApi', ['$httpParamSerializer', 'Ajax', function ($httpParamSerializer, Ajax) {
        var urlStyles = {
            restful : {
                index: '{apiurl}?page={page}',
                create: '{apiurl}/create',
                store: '{apiurl}',
                show: '{apiurl}/{id}',
                edit: '{apiurl}/{id}/edit',
                update: '{apiurl}/{id}',
                delete: '{apiurl}/{id}',
                custom : '{apiurl}/{action}'
            },
            parallelism : {
                index: '{apiurl}/list?page={page}',
                create: '{apiurl}/create',
                store: '{apiurl}/create',
                show: '{apiurl}/show?id={id}',
                edit: '{apiurl}/edit?id={id}',
                update: '{apiurl}/edit?id={id}',
                delete: '{apiurl}/delete?id={id}',
                custom: '{apiurl}/{action}'
            },
            lwap : {
                index: '{apiurl}?act=_loadlist_&page={page}',
                store: '{apiurl}?act=_create_',
                show: '{apiurl}?act=_show_&id={id}',
                edit: '{apiurl}?act=_loadedit_&id={id}',
                update: '{apiurl}?act=_modify_&id={id}',
                delete: '{apiurl}?act=_delete_&id={id}',
                opposite : '{apiurl}?act=_opposite_&id={id}&field={field}',
                custom: '{apiurl}?act={action}'
            }
        };

        function Api(config) {//api_url, url_style, url_params, ajax
            var that = this;
            var ajax = config.ajax || Ajax.make();
            this.config = function(){
                if (arguments.length) {
                    var arg0 = arguments[0];
                    if ( typeof(arg0) == 'string' ) {
                        return config[arg0];
                    } else {
                        config = arguments[0];
                    }
                } else {
                    return config;
                }
            },
            this.debug = function(){
                ajax.debug();
                return this;
            },
            this.buildUrl = function(action, params) {
                var replaces = angular.extend({}, params);
                replaces['apiurl'] = config.api_url;
                replaces['params'] = (function(){
                    var params = angular.extend({}, params || {}, config.url_params || {});
                    return $httpParamSerializer(params);
                })();
                var apiurl = urlStyles[config.url_style][action];
                var link = apiurl.indexOf('?') > 0 ? '&' : '?';

                if (replaces['params']) apiurl += link + '{params}';
                for (var k in replaces) {
                    apiurl = apiurl.replace('{' + k + '}', replaces[k]);
                }
                return apiurl;
            },
            this.loading = function(is){
                ajax.loading(is);
                return this;
            },
            this.show = function(id, params){
                var url = this.buildUrl('show', {id:id});
                return ajax.get(url, params);
            },
            this.list = function(page, params){
                var url = this.buildUrl('index', {page:page || 1});
                return ajax.get(url, params);
            },
            this.edit = function(datId, params){
                var url = this.buildUrl('edit', {id:datId});
                return ajax.get(url, params);
            },
            this.create = function(data, params){
                var url = this.buildUrl('store');
                return ajax.post(url, data, params);
            },
            this.update = function(datId, data, params){
                var url = this.buildUrl('update', {id:datId});
                return ajax.put(url, data, params);
            },
            this.delete = function(datIds, params){
                if (angular.isArray(datIds)) datIds = datIds.join(',');
                var url = this.buildUrl('delete', {id:datIds});
                return ajax.delete(url, params);
            },
            this.opposite = function(datId, field) {
                var url = this.buildUrl('opposite', {id:datId, field:field});
                return ajax.get(url);
            },
            this.get = function(action, params) {
                var url = this.buildUrl('custom', {action:action});
                return ajax.get(url, params);
            },
            this.post = function(action, data, params) {
                var url = this.buildUrl('custom', {action:action});
                return ajax.post(url, data, params);
            };
        }

        return {
            make : function(config){
                return new Api(config);
            }
        };
    }]);