angular.module('Landers.model', [])
    .directive('checklistModel', ['$parse', '$compile', function($parse, $compile) {
        // contains
        function contains(arr, item, comparator) {
            if (angular.isArray(arr)) {
                for (var i = arr.length; i--;) {
                    if (comparator(arr[i], item)) {
                        return true;
                    }
                }
            }
            return false;
        }

        // add
        function add(arr, item, comparator) {
            arr = angular.isArray(arr) ? arr : [];
            if (!contains(arr, item, comparator)) {
                arr.push(item);
            }
            return arr;
        }

        // remove
        function remove(arr, item, comparator) {
            if (angular.isArray(arr)) {
                for (var i = arr.length; i--;) {
                    if (comparator(arr[i], item)) {
                        arr.splice(i, 1);
                        break;
                    }
                }
            }
            return arr;
        }

        // http://stackoverflow.com/a/19228302/1458162
        function postLinkFn(scope, elem, attrs) {
            // exclude recursion, but still keep the model
            var checklistModel = attrs.checklistModel;
            attrs.$set("checklistModel", null);
            // compile with `ng-model` pointing to `checked`
            $compile(elem)(scope);
            attrs.$set("checklistModel", checklistModel);

            // getter for original model
            var checklistModelGetter = $parse(checklistModel);
            var checklistChange = $parse(attrs.checklistChange);
            var checklistBeforeChange = $parse(attrs.checklistBeforeChange);
            var ngModelGetter = $parse(attrs.ngModel);



            var comparator = angular.equals;

            if (attrs.hasOwnProperty('checklistComparator')) {
                if (attrs.checklistComparator[0] == '.') {
                    var comparatorExpression = attrs.checklistComparator.substring(1);
                    comparator = function(a, b) {
                        return a[comparatorExpression] === b[comparatorExpression];
                    };

                } else {
                    comparator = $parse(attrs.checklistComparator)(scope.$parent);
                }
            }

            // watch UI checked change
            scope.$watch(attrs.ngModel, function(newValue, oldValue) {
                if (newValue === oldValue) {
                    return;
                }

                if (checklistBeforeChange && (checklistBeforeChange(scope) === false)) {
                    ngModelGetter.assign(scope, contains(checklistModelGetter(scope.$parent), getChecklistValue(), comparator));
                    return;
                }

                setValueInChecklistModel(getChecklistValue(), newValue);

                if (checklistChange) {
                    checklistChange(scope);
                }
            });

            // watches for value change of checklistValue (Credit to @blingerson)
            scope.$watch(getChecklistValue, function(newValue, oldValue) {
                if (newValue != oldValue && angular.isDefined(oldValue) && scope[attrs.ngModel] === true) {
                    var current = checklistModelGetter(scope.$parent);
                    checklistModelGetter.assign(scope.$parent, remove(current, oldValue, comparator));
                    checklistModelGetter.assign(scope.$parent, add(current, newValue, comparator));
                }
            });

            function getChecklistValue() {
                return attrs.checklistValue ? $parse(attrs.checklistValue)(scope.$parent) : attrs.value;
            }

            function setValueInChecklistModel(value, checked) {
                var current = checklistModelGetter(scope.$parent);
                if (angular.isFunction(checklistModelGetter.assign)) {
                    if (checked === true) {
                        checklistModelGetter.assign(scope.$parent, add(current, value, comparator));
                    } else {
                        checklistModelGetter.assign(scope.$parent, remove(current, value, comparator));
                    }
                }

            }

            // declare one function to be used for both $watch functions
            function setChecked(newArr, oldArr) {
                if (checklistBeforeChange && (checklistBeforeChange(scope) === false)) {
                    setValueInChecklistModel(getChecklistValue(), ngModelGetter(scope));
                    return;
                }
                ngModelGetter.assign(scope, contains(newArr, getChecklistValue(), comparator));
            }

            // watch original model change
            // use the faster $watchCollection method if it's available
            if (angular.isFunction(scope.$parent.$watchCollection)) {
                scope.$parent.$watchCollection(checklistModel, setChecked);
            } else {
                scope.$parent.$watch(checklistModel, setChecked, true);
            }
        }

        return {
            restrict: 'A',
            priority: 1000,
            terminal: true,
            scope: true,
            compile: function(tElement, tAttrs) {

                if (!tAttrs.checklistValue && !tAttrs.value) {
                    throw 'You should provide `value` or `checklist-value`.';
                }

                // by default ngModel is 'checked', so we set it if not specified
                if (!tAttrs.ngModel) {
                    // local scope var storing individual checkbox model
                    tAttrs.$set("ngModel", "checked");
                }

                return postLinkFn;
            }
        };
    }])
    .directive('uiAlertError', ['Flat', function(Flat){
        return {
            restrict : 'A',
            link : function($scope, $ele, $attrs){
                var datKey = $attrs['uiAlertError'];
                $ele.addClass('alert alert-danger').hide();
                $scope.$watch(datKey, function(newValue){
                    if (newValue) {
                        var html = '<i class="fa fa-minus-circle"></i> ' + newValue;
                        $ele.html(html).show();
                    } else {
                        $ele.hide();
                    }
                });
                var $form = $ele.closest('form');
                $form.find('input, select, textarea').change(function(){
                    var ele = $(this);
                    if (!ele.val()) return;
                    var ngmodel = ele.attr('ng-model');
                    var arr = ngmodel.split('.');
                    arr.splice(1, 0, '$errors');
                    var datkey = arr.join('.');
                    Flat.set($scope, datkey, '');
                    if (!$scope.$$phase) $scope.$apply();
                });
            }
        };
    }])
    .directive('listStatus', ['$templateRequest', '$compile', 'Flat', 'Helpers',  function($templateRequest, $compile, Flat, Helpers){
        return {
            restrict : 'A',
            link : function($scope, $ele, $attrs){
                $ele.addClass('list-status');
                var myScope = Helpers.seekScope($scope, $attrs['useScope']);

                var modelKey = $attrs['modelKey'], html = '';
                var datakey1 = [modelKey, '$list', 'data'].join('.');
                var icon1 = '<i class="glyphicon glyphicon-remove-sign"></i> ';
                var icon2 = '<i class="glyphicon glyphicon-info-sign"></i>';

                myScope.$watch(datakey1, function(newValue){
                    var html;
                    if (newValue) {
                        if ( !newValue.length ) {
                            var datakey2 = [modelKey, '$errors.$list'].join('.');
                            var error_message = Flat.get(myScope, datakey2);
                            if ( !error_message ) {
                                datakey3 = [modelKey, '$search'].join('.');
                                var htmlTpl = '<div class="alert alert-warning pv20">%s</div>';
                                var datakey4 = [modelKey, '$name'].join('.');
                                var appname = Flat.get(myScope, datakey4);

                                var searchData = Flat.get(myScope, datakey3);
                                if ( searchData && Landers.utils.json.count(searchData) ) {
                                    html = icon1 + ' 没有检索到符合条件的' + appname + '。您可以尝试其他搜索条件或<a class="highlight hand" ng-click="$events.resetSearch()">重置搜索</a> ';
                                    html = Landers.utils.str.sprintf(htmlTpl, html);
                                } else {
                                    var partial = $attrs['partialNone'];
                                    var message = $attrs['messageNone'];
                                    if (partial) {
                                        var partialUrl = myScope.$partial(partial);
                                        $templateRequest(partialUrl).then(function(html){
                                            html = Landers.utils.str.sprintf(htmlTpl, html);
                                            $ele.fadeOut(300, function(){
                                                $ele.html($compile(html)(myScope)).fadeIn(300);
                                            });
                                        });
                                        return;

                                    } else if (message) {
                                        html = message;
                                    } else {
                                        html = '暂无' + appname + '。';
                                    }
                                    html = Landers.utils.str.sprintf(htmlTpl, icon1 + html);
                                }
                            } else {
                                html = '<div class="alert alert-danger pv20">' + icon2 + ' ' + error_message + ' </div>';
                            }
                            $ele.fadeOut(300, function(){
                                $ele.html($compile(html)(myScope)).fadeIn(300);
                            });
                        } else {
                            $ele.text('').fadeOut(300);
                        }
                    } else {
                        html = '<div class="alert alert-default pv20"><i class="glyphicon glyphicon-cog glyphicon-spin fa fa-cog fa-spin"></i> loading... </div>';
                        $ele.html($compile(html)(myScope)).hide().fadeIn(500);
                    }
                }, true);
            }
        };
    }])
    .factory('AngularModel', ['$templateRequest', '$timeout', 'List', 'Flat', 'Dialog', 'AngularApi', 'Helpers', function ($templateRequest, $timeout, List, Flat, Dialog, AngularApi, Helpers) {
        function Model(scope, config) { // api_url, url_style, url_params, lister, key
            var that = this, watchers = [];

            //定义模型标识
            if (!config.key) throw ('未定义key字段');
            this.key = config.key;
            this.name = config.name;

            //将当前模型数据容器
            var modelkey = '$' + that.key;
            scope[modelkey] = {
                '$name' : config.name,
                '$list' : {
                    'data' : null,
                    'page' : null
                },
                '$create' : {},
                '$detail' : null,
                '$items' : {
                    selected:[],
                    selectedRows:[],
                },
                '$edit' : {},
                '$form' : {},
                '$update' : {},
                '$modify' : {},
                '$search' : null,
                '$errors' : {
                    '$list' : null,
                    '$edit' : null,
                    '$form' : null,
                }
            };

            var is_api_result = function(rspn) {
                if ( !angular.isObject(rspn)) return false;
                return  'success' in rspn &&
                        'message' in rspn &&
                        'data' in rspn;
            };

            this.extendData = function(key, extdata, subkey) {
                var data = that.getData(key, subkey);
                Flat.extend(data, extdata);
                if (!scope.$$phase) scope.$apply();
                return ret;
            };

            this.setData = function(key, data, subkey) {
                var datakey = modelkey + '.' + key;
                Flat.set(scope, datakey, data, subkey);
                if (!scope.$$phase) scope.$apply();
                return ret;
            };

            this.getData = function(key, subkey) {
                var datakey = modelkey + '.' + key;
                return Flat.get(scope, datakey, subkey);
            };

            this.watchers = [];
            this.watch = function(object, callback) {
                if (angular.isString(object)) {
                    object = modelkey + '.' + object;
                }
                that.watchers.push(scope.$watch(object, callback));
            };
            this.destroyWatchers = function(){
                for(var i=0; i<that.watchers.length; i++) {
                    that.watchers[i]();
                }
            };

            this.clearErrors = function(key) {
                that.setData('$errors', {}, key);
            };

            this.setErrors = function(key, data, message) {
                for (var k in data) {
                    if (angular.isArray(data[k])) {
                        data[k] = data[k][0];
                    }
                }
                that.setData('$errors', data, key);

                var isMsg;
                if ( message === true || message === false) {
                    isMsg = message;
                    message = '提交的数据还存在不足！';
                } else {
                    isMsg = true;
                }

                if (isMsg && message) Landers.response.msgbox({
                    success : false,
                    message : message
                });

                return message;
            };

            //定义钩子模块
            this.hooks = {
                data : {
                    list:{
                        before:[],
                        after:[]
                    },
                    edit:{
                        onload:{
                            before:[],
                            after:[]
                        },
                        submit:{
                            before:[],
                            after:[]
                        }
                    },
                    close:[]
                },
                registe: function(key, fns) {
                    var _this = this;
                    if (!angular.isArray(fns)) fns = [fns];
                    angular.forEach(fns, function (fn){
                        if (angular.isFunction(fn)) {
                            Flat.push(_this.data, key, fn);
                        }
                    });
                },
                registeOnce: function(key, fn) {
                    if ( Flat.get(this.data, key).length ) return;
                    this.registe( key, fn);
                },
                run : function(key, params){
                    var _this = this;
                    var funs = Flat.get(_this.data, key);
                    angular.forEach(funs, function (fn){fn(params);});
                }
            };

            //创建api与服务端通信对象
            config['api_url'] = config['api_url'] || ('/' + config.key);
            var api_config = Landers.utils.json.slice(config, [
                'api_url', 'url_style', 'url_params', 'ajax'
            ]);
            this.api = AngularApi.make(api_config);

            //定义列表操作对象
            // laravel输出分页时，数据位于 data字段中
            this.lister = config.lister || List.make(scope, modelkey + '.$list.data', null);

            //模板Url容器
            this.templateUrls = this.templateUrls || {};

            //对话框模块
            this.dialog = {
                dialogs : [],
                attachConfig : function(_dialog_config){
                    var dialog_config = angular.extend({}, _dialog_config);
                    if (_dialog_config.event) {
                        that.listitem.active(_dialog_config.event, true);
                        dialog_config.onclose = function(){
                            that.listitem.active(_dialog_config.event, false);
                            _dialog_config.onclose && _dialog_config.onclose.call(this);
                        };
                        delete dialog_config.event;
                    }
                    return dialog_config;
                },
                show : function(_dialog_config){
                    var dialog_config = this.attachConfig(_dialog_config);
                    var _this = this;
                    var _show = function(){
                        var dialog = Dialog.show(scope, dialog_config);
                        _this.dialogs.push(dialog);
                    };
                    var partial = dialog_config.tplkey || dialog_config.partial;
                    if (partial) {
                        delete dialog_config.tplkey;
                        delete dialog_config.partial;
                        if (!scope.$partial) throw('未定义scope.$partial');
                        $templateRequest(scope.$partial(partial)).then(function(html){
                            dialog_config.html = html;
                            _show();
                        });
                    } else {
                        _show();
                    }
                }
            };

            //列表操作模块
            this.list = {
                // selectedData : function(){
                //     var data = that.getData('$list.data');
                //     var ret = [];
                //     for (var i=0; i<data.length; i++) {
                //         if (data[i].checked) ret.push(data[i]);
                //     }
                //     return ret;
                // },
                put:function(data) {
                    that.setData('$list', data);
                },
                data:function(data){
                    return that.lister.data(data);
                },
                clear:function(){
                    that.lister.clear();
                    that.listitem.selectAll(false);
                    that.setData('$list', {});
                    that.setData('$errors.$list', null);
                    return this;
                },
                doShow:function(ajax, opts){
                    opts = opts || {};
                    ajax.success(function(rspn){
                        if (angular.isArray(rspn)) {
                             //RESTFUL 返回纯数据
                            that.setData('$list.data', rspn);
                        } else {
                            Landers.response.msgbox(rspn);

                            // 返回ApiResult 对象包
                            if (angular.isArray(rspn.data)) {
                                that.setData('$list.data', rspn.data);
                            } else {
                                if (rspn.data.$list) {
                                    that.setData('$list', rspn.data.$list);
                                } else if (rspn.data.data) {
                                    if (rspn.data.page) {
                                        that.setData('$list.data', rspn.data.data);
                                        that.setData('$list.page', rspn.data.page);
                                    } else {
                                        that.setData('$list', rspn.data);
                                    }
                                } else {
                                    // doNothing
                                }
                            }

                        }
                        that.hooks.run('list.after', rspn);
                        opts.success = opts.success || opts.callback;
                        opts.success && opts.success(rspn);
                    })
                    .error(function(rspn){
                        opts.error && opts.error(rspn);
                        try {
                            that.setData('$errors.$list', rspn.message);
                            that.setData('$list', {data : [], page:{}});
                        } catch (e) {}
                    });
                },
                show:function(opts){
                    var _this = this;
                    opts = opts || {};
                    that.hooks.run('list.before');

                    var $search = that.getData('$search') || {};
                    for (var k in $search) {
                        if ( k.indexOf('$$$') === 0 ) delete $search.k;
                    }

                    opts.params = angular.extend({}, opts.params, {
                        search:angular.toJson($search)
                    });
                    var doShow = function() {
                        var ajax = that.api.loading(opts.loading)
                            .list(opts.page, opts.params);

                        _this.doShow(ajax);

                        opts.finally && ajax.finally(opts.finally);

                        return ajax;
                    };

                    this.clear();
                    if (opts.callbackBefore) {
                        return opts.callbackBefore(doShow);
                    } else {
                        return doShow();
                    }
                },
                popup:function(_dialog_config, callback_onload){
                    var _this = this;
                    var dialog_config = that.dialog.attachConfig(_dialog_config);
                    dialog_config.onload = function(){
                        var dialog = this;
                        _this.show({callback:function(rspn){
                            callback_onload && callback_onload.call(dialog, rspn);
                        }});
                        _dialog_config.onload && _dialog_config.onload.call(this);
                        setTimeout(function(){dialog.position();}, 300);
                    };

                    dialog_config = angular.extend({}, {
                        tplkey : dialog_config.partial || dialog_config.tplkey || 'lister',
                        scroll : true,
                        height : '100%',
                        no : true
                    }, dialog_config);
                    that.dialog.show(dialog_config);
                },
                refresh:function(callback){
                    var opts = {
                        page : that.getData('$list.page.current') || 1,
                        params : {
                            search:angular.toJson(that.getData('$search'))
                        },
                        callback : callback
                    };
                    this.show(opts);
                },
                ifRefresh : function(){
                    if (!that.lister.count()) this.refresh();
                },
                preAppend : function(data){
                    return that.lister.preAppend(data);
                },
                append : function(data) {
                    return that.lister.append(data);
                }
            };

            //列表横操作模块
            this.item = this.listitem = {
                show:function(id, params){
                    return that.api.show(id, params)
                        .success(function(rspn){
                            that.setData('$detail', rspn.data);
                        })
                },
                data:function($event, key) {
                    return that.lister.item.data($event, key);
                },
                swap:function(item1, item2){
                    var params = {previd:item1.id, nextid:item2.id};
                    return that.api.get('swap_order', params)
                        .success(function(){
                            var index1 = that.lister.mapIndex(item1.id);
                            var index2 = that.lister.mapIndex(item2.id);
                            that.lister.item.swap(index1, index2);
                        })
                        .error(function(rspn){
                            Landers.response.msgbox(rspn);
                        });
                },
                moveUp:function(item1) {
                    var index1 = that.lister.mapIndex(item1.id), index2;
                    if (index1 === 0) {
                        return alert('当前项已经处于最前了');
                    } else {
                        index2 = index1 - 1;
                    }
                    var item2 = this.data(index2);
                    this.swap(item1, item2);
                },
                moveDown:function(item1){
                    var index1 = that.lister.mapIndex(item1.id), index2;
                    if (index1 == that.list.data().length) {
                        return alert('当前项已经处于最后了');
                    } else {
                        index2 = index1 + 1;
                    }
                    var item2 = this.data(index2);
                    this.swap(item1, item2);
                },
                count:function(){
                    var data = that.lister.data();
                    return data ? that.lister.data().length : -1;
                },
                update:function($event, dat, item){
                    if ( item ) {
                        angular.extend(item, dat);
                    } else {
                        that.lister.item.update($event, dat);
                    }
                },
                active:function($event, is_active){
                    return that.lister.item.active($event, is_active);
                },
                select:function($event, is_check){
                    return that.lister.item.select($event, is_check);
                },
                selectAll : function(bool){
                    var data = that.lister.data();
                    if ( !data ) return false;
                    data = bool ? data.map(function(item){return item.id || item.key;}) : [];
                    that.setData('$items', data, 'selected');
                },
                getSelectedId : function(alert_message){
                    if (alert_message === true) {
                        alert_message = '未选择目标数据！';
                    }
                    var ids = that.getData('$items', 'selected');
                    if ((!ids || !ids.length) && alert_message) {
                        alert(alert_message);
                        return null;
                    }
                    return ids;
                },
                remove:function(ids, from_lister) {
                    var _this = this;
                    if (!angular.isArray(ids)) {
                        ids = [ids];
                    }
                    var lister = from_lister || that.lister;
                    angular.forEach(ids, function(id){
                        var index = lister.mapIndex(id);
                        lister.item.remove(index);
                    });
                    if (!_this.count()) that.list.refresh();

                    var total = that.getData('$list.page.total');
                    if (total) {
                        that.setData('$list.page.total', total - ids.length);
                    }
                },
                deleteById:function(ids, from_lister){
                    var _this = this;
                    return that.api.delete(ids)
                        .success(function(rspn){
                            _this.remove(ids, from_lister);
                            that.list.ifRefresh();
                        });
                },
                delete:function(item, from_lister){
                    var _this = this;
                    return _this.deleteById(item.id, from_lister);
                },
                deleteSelected:function(callback){
                    var ids = this.getSelectedId(true);
                    if (!ids) return;
                    if ( !confirm('确定要删除选择的数据？')) return;
                    return this.deleteById(ids)
                        .success(function(rspn){
                            that.setData('$items', [], 'selected');
                            callback && callback(rspn);
                        });
                }
            };

            this.addedit = {
                edit : function(item, onLoad, params, hook){
                    return that.api.edit(item.id, params || {})
                               .success(function(rspn){
                                    var editData = (function(){
                                        if ( !rspn.data ) return rspn;
                                        if ( rspn.data.$edit ) return rspn.data.$edit;
                                        return rspn.data;
                                    })();

                                    if (editData) that.setData('$edit', editData);
                                    onLoad && onLoad(rspn);
                                    hook && hook(rspn);
                               });
                },
                update : function( item, onSubmit, hook){
                    var data = that.getData('$edit');
                    return that.api.update(item.id, data)
                        .success(function(rspn){
                            var updateData = (function(){
                                if ( !rspn.data ) return rspn;
                                if ( rspn.data.$modify ) return rspn.data.$modify;
                                if ( rspn.data.$update ) return rspn.data.$update;
                                return rspn.data;
                            });
                            onSubmit && onSubmit(rspn);
                            if (updateData) angular.extend(item, updateData);
                            hook && hook(rspn);
                        });
                },
                create : function(onSubmit, hook){
                    var data = that.getData('$edit');
                    return that.api.create(data)
                        .success(function(rspn){
                            onSubmit && onSubmit(rspn);
                            var page_info = that.getData('$list.page') || {};
                            if ( '$total' in  page_info ) {
                                that.list.refresh();
                            } else {
                                if (rspn.data && rspn.data.$create) {
                                    that.list.append(rspn.data.$create || rspn.data);
                                }
                                hook && hook(rspn);
                            }
                        });
                },
                show : function($event, item, callback){
                    var action = item ?
                        {key : 'edit', text : '编辑', is_edit : true, datId: item.id, event:$event} :
                        {key : 'create', text : '添加', is_edit : false, datId:null, event:$event };

                    var _this = this;

                    var doLoad = function(onLoad, params){
                        that.setData('$edit', {});
                        that.setData('$errors.$edit', null);
                        that.hooks.run('edit.onload.before', {action:action});
                        if (action.is_edit) {
                            return _this.edit(item, onLoad, params, function(rspn){
                                that.hooks.run('edit.onload.after', {action:action, rspn:rspn});
                            });
                        } else {
                            onLoad && onLoad({});
                            that.hooks.run('edit.onload.after', {action:action});
                        }
                    };
                    var doSubmit = function(onSubmit){
                        that.hooks.run('edit.submit.before', {action:action});
                        return  action.is_edit ?
                                _this.update(item, onSubmit, function(rspn){
                                    that.hooks.run('edit.submit.after', {action:action, rspn:rspn});
                                }) :
                                _this.create( onSubmit, function(rspn){
                                    that.hooks.run('edit.submit.after', {action:action, rspn:rspn});
                                });
                    };

                    callback(action, doLoad, doSubmit);
                },
                popup : function($event, item, _dialog_config) {
                    var dialog_config = that.dialog.attachConfig(_dialog_config);
                    var partial = dialog_config.partial || dialog_config.tplkey;
                    delete dialog_config.partial;
                    delete dialog_config.tplkey;

                    var _this = this, dialog;
                    var templateUrl = partial ? scope.$partial(partial) :
                                      (that.templateUrls.form || scope.$partial('form'));

                    $templateRequest(templateUrl).then(function(html){
                        _this.show($event, item, function (action, doLoad, doSubmit){
                            if (action.is_edit) {
                                dialog_config.loading = {size:'small'};
                            }
                            dialog_config = angular.extend({}, dialog_config, {
                                event : $event,
                                title : action.text + (dialog_config.name || that.name),
                                html: html,
                                onload : function(){
                                    dialog = this;
                                    var onBeforeLoad = dialog_config.onBeforeLoad;
                                    var _onAfterLoad = function(rspn){
                                        var onAfterLoad = dialog_config.onAfterLoad;
                                        onAfterLoad && onAfterLoad.call(that, action, dialog, rspn);

                                        if ( is_api_result(rspn) ) dialog.done(rspn); else dialog.done();
                                    };
                                    var _doLoad = function(){
                                        doLoad(_onAfterLoad);
                                    };
                                    $timeout(function() {
                                        if ( onBeforeLoad ) {
                                            onBeforeLoad(action, dialog, _doLoad);
                                        } else {
                                            _doLoad();
                                        }
                                    }, 100);
                                },
                                yes : {
                                    text:'保存', fn:function(window, button){
                                        Helpers.disable($(button));
                                        var onBeforeSubmit = dialog_config.onBeforeSubmit;
                                        var onAfterSubmit = dialog_config.onAfterSubmit;
                                        var _doSubmit = function(){
                                            doSubmit()
                                                .success(function(rspn){
                                                    Helpers.enable($(button));
                                                    dialog.done(rspn);
                                                    if (action.is_edit) {
                                                        if (rspn.data) {
                                                            var updateData = rspn.data.$modify || rspn.data.$update;
                                                            if ( updateData ) {
                                                                that.listitem.update($event, updateData, item);
                                                            } else {
                                                                that.list.refresh();
                                                            }
                                                        } else {
                                                            if ( angular.isObject(rspn) ) {
                                                                that.listitem.update($event, rspn, item);
                                                            } else if (rspn === true) {
                                                                var data = that.getData('$edit');
                                                                that.listitem.update($event, data, item);
                                                            } else {
                                                                that.list.refresh();
                                                            }
                                                        }
                                                    } else {
                                                        if ( is_api_result(rspn) ) {
                                                            var createData = rspn.data.$modify || rspn.data.$update;
                                                            if ( createData ) {
                                                                item = angular.extend(createData);
                                                                that.list.preAppend(item);
                                                            } else {
                                                                that.list.refresh();
                                                            }
                                                        } else {
                                                            if ( angular.isObject(rspn) ) {
                                                                item = angular.extend(rspn);
                                                                that.list.preAppend(item);
                                                            } else if (rspn === true) {
                                                                item = angular.extend(that.getData('$edit'));
                                                                that.list.preAppend(item);
                                                            } else {
                                                                that.list.refresh();
                                                            }
                                                        }
                                                    }
                                                    onAfterSubmit && onAfterSubmit(action, dialog, rspn, item);
                                                    if (!rspn.message) dialog.close();
                                                    dialog.btn().get('取消').text('关闭');
                                                })
                                                .error(function(rspn){
                                                    if ( angular.isObject(rspn.errors) ) {
                                                        rspn.message = rspn.message || that.setErrors('$edit', rspn.errors, false);
                                                    }
                                                    dialog.done(rspn);
                                                    Helpers.enable($(button));
                                                });
                                        };


                                        dialog.loading({
                                            text : '正在提交数据...',
                                            size : 'long'
                                        }, function(callback_result){
                                            $timeout(function() {
                                                if ( onBeforeSubmit ) {
                                                    onBeforeSubmit(action, dialog, _doSubmit);
                                                } else {
                                                    _doSubmit();
                                                }
                                            }, 100);
                                        });
                                        return false;
                                    }
                                },
                                no : '取消'
                            });
                            dialog_config.onclose = dialog_config.onclose || function(){
                                scope.$broadcast('destroy_ueditor');
                                that.setData('$edit', {});
                            };
                            that.dialog.show(dialog_config);
                        });
                    });
                }
            };


            //新初始化$events, （注：必须不能用angular.extend来扩展，因为scope可能某子controller并继承了父controller）
            if ( !config.vice ) {
                scope.$events = {
                    selectAll : function($event) {
                        $event = angular.element($event.target);
                        var checked = $event.prop('checked');
                        that.listitem.selectAll(checked);
                    },
                    refreshList : function(){
                        that.list.refresh();
                    },
                    deleteItem : function($event, item, from_lister) {
                        Helpers.disable($event);
                        that.listitem.active($event, true);

                        if (item) item.$busy = true;
                        if (!confirm('确定要删除吗？')) {
                            that.listitem.active($event, false);
                            Helpers.enable($event);
                            if (item) item.$busy = false;
                            return;
                        }
                        return that.listitem.delete(item, from_lister)
                            .success(function(rspn){
                                Landers.response.msgbox(rspn);
                            })
                            .error(function(rspn){
                                Landers.response.msgbox(rspn);
                                if (item) item.$busy = false;
                                Helpers.enable($event);
                            });
                    },
                    deleteSelected : function() {
                        that.listitem.deleteSelected();
                    },
                    search : function(page){
                        var opts = {page : page};
                        var dat = that.getData('$search') || {};
                        if (dat._keyword_ || dat._keyword_by_) {
                            if (!dat._keyword_) {
                                return alert('未输入搜索关键词');
                            }
                            if (!dat._keyword_by_) {
                                return alert('未选择搜索关键词依据');
                            }
                        }
                        that.list.show(opts);
                    },
                    resetSearch : function(){
                        that.setData('$search', {});
                        scope.$emit('reset-search');
                        scope.$broadcast('reset-search');
                        that.list.show();
                    },
                    moveUp : function($event, item) {
                        that.listitem.moveUp(item);
                    },
                    moveDown : function($event, item) {
                        that.listitem.moveDown(item);
                    },
                    opposite : function($event, item, prop){
                        return that.api.opposite(item.id, prop)
                            .success(function(rspn){
                                item[prop] = rspn.data[prop];
                            });
                    }
                };
            }

            scope.$on('$destroy', function(){
                for (var i=0; i<watchers.length; i++) watches[i]();

                // Landers.dialog.clear && Landers.dialog.clear();
                angular.forEach(that.dialog.dialogs, function(dialog){
                    if (dialog && dialog.close) {
                        dialog.close();
                    }
                });

                Landers.response.hide();
            });
        }

        return {
            make:function(scope, config){
                return new Model(scope, config);
            }
        };
    }]);